<?php

/**
 * The functionalities for MPT images generation
 *
 * @package    Magic_Post_Thumbnail
 * @subpackage Magic_Post_Thumbnail/admin
 * @author     Magic Post Thumbnail <contact@magic-post-thumbnail.com>
 */
class Magic_Post_Thumbnail_Generation extends Magic_Post_Thumbnail_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    4.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    4.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    4.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name 	= $plugin_name;
		$this->version 		= $version;

		// Ajax calls
		add_action( 'wp_ajax_nopriv_generate_image',array( &$this, 'MPT_ajax_call' ) );
		add_action( 'wp_ajax_generate_image', array( &$this, 'MPT_ajax_call' ) );

		$main_settings	= wp_parse_args( get_option( 'MPT_plugin_main_settings' ), $this->MPT_default_options_main_settings( FALSE ) );

		// Enable save_post hook
		if( isset( $main_settings['enable_save_post_hook'] ) && ( 'enable' == $main_settings['enable_save_post_hook'] ) ) {

			add_action( 'save_post', array( &$this, 'MPT_check_post_type' ), 10, 3 );
		}

	}


	/**
	 * Ajax call for bulk generation
	 *
	 * @since    4.0.0
	*/
	public function MPT_ajax_call() {

	    $post_ids = array_map( 'absint', json_decode( $_POST['ids_mpt_generation'] ) );

	    // Security checks & Check if user has rights
	    if ( ! current_user_can('mpt_manage') || false === wp_verify_nonce( $_POST['nonce'], 'ajax_nonce_magic_post_thumbnail' ) ) {
	        wp_send_json_error();
	    }

	    if( !isset( $_POST['ids_mpt_generation'] ) )
	        return false;

	    $count = count( $post_ids );

	    foreach ( $post_ids as $key => $val )
	        $ids[ $key+1 ] = $val;

	    $a  = (int)$_POST['a'];
	    $id = $ids[$a];

	    $main_settings = get_option( 'MPT_plugin_main_settings' );
	    if( isset( $main_settings['rewrite_featured'] ) && ( $main_settings['rewrite_featured'] == true ) )  {
	            $rewrite_featured = true;
	    } else {
	            $rewrite_featured = false;
	    }

	    $speed = '500';

	    // Image location
	    $image_location  = ( ! empty( $main_settings['image_location'] ) ) ? $main_settings['image_location']  : 'featured';

	    if ( "featured" !== $image_location ) {
				if( $this->mpt_freemius()->is__premium_only() ) {
	
					if ( $this->mpt_freemius()->can_use_premium_code() ) {
							
						// Check if image generation into content is already done
						$content_post = get_post( $id );
						$image_generated = strpos( $content_post->post_content, 'mpt-img' ) ? true : false;
						if( ( $image_generated ) && ( false == $rewrite_featured ) ) {
								$status = 'already-done';
						} elseif( ( $image_generated ) && ( true == $rewrite_featured ) ) {
								$status = 'no-rewrite';
						} elseif( ( false == $image_generated || $rewrite_featured == true ) && $id != 0 ) {
								$MPT_return = $this->MPT_create_thumb( $id, '0', '0', '0', $rewrite_featured );
								if( $MPT_return == null ) {
										$status = 'failed';
								} else {
										$status = 'successful';
										$speed = '500';
								}
						} else {
								$status = 'error';
						}

					}
				}
	    } else {
	      if( has_post_thumbnail( $id ) && ( $rewrite_featured == false ) ) {
	              $status = 'already-done';
	      } elseif( (!has_post_thumbnail( $id ) || $rewrite_featured == true ) && $id != 0 ) {
	              $MPT_return = $this->MPT_create_thumb( $id, '0', '0', '0', $rewrite_featured );
	              if( $MPT_return == null ) {
	                      $status = 'failed';
	              } else {
	                      $status = 'successful';
	                      $speed = '500';
	              }
	      } else {
	              $status = 'error';
	      }
	    }

	    $percent = ( 100*$a )/$count;

	    $compatibility = wp_parse_args( get_option( 'MPT_plugin_compatibility_settings' ), $this->MPT_default_options_compatibility_settings( TRUE ) );
	    $img = '';

	    if ( true == $compatibility['enable_FIFU'] && is_plugin_active( 'featured-image-from-url/featured-image-from-url.php' ) ) {
			// FIFU Plugin
			if( $this->mpt_freemius()->is__premium_only() ) {
				if ( $this->mpt_freemius()->can_use_premium_code() ) {
					$img_url    = get_the_post_thumbnail_url( $id );
					$img        = $img_url;
					$fimg       = '<img src="' . $img_url . '" width="70" height="70" />';
					$datas['postimagediv']   = _wp_post_thumbnail_html( $MPT_return, $id );
				}
			}
	    } elseif( ( ( $status == 'already-done' ) || ( $status == 'successful' ) ) && ( ! empty( $MPT_return ) ) ) {
	        // Generation successful
	        $new_image  = wp_get_attachment_image_src( $MPT_return, array( 70, 70 ) );
	        $fimg       = '<a class="generated-img" target="_blank" href="' . admin_url() . 'upload.php?item=' . $MPT_return . '"><img src="'. $new_image[0] .'" width="70" height="70" /></a>';

	        $datas['thumbnail_id']   = $MPT_return;
	        $datas['postimagediv']   = _wp_post_thumbnail_html( $MPT_return, $id );

	    } elseif( ( $status == 'already-done' ) && ( "featured" === $image_location ) ) {
          $fimg = '<a class="generated-img" target="_blank" href="' . admin_url() . 'upload.php?item=' . get_post_thumbnail_id( $id ) . '">'. get_the_post_thumbnail( $id, array( '70', '70' ) ) .'</a>';
	    } else {
          $fimg = '<img src="' . plugins_url( 'img/no-image.jpg', __FILE__ ) . '" />';
	    }

	    $datas['percent']        = $percent;
	    $datas['id']             = $id;
	    $datas['status']         = $status;
	    $datas['img']            = $img;
	    $datas['fimg']           = $fimg;
	    $datas['speed']          = $speed;

	    // Send data to JavaScript
	    if ( ! empty( $datas['id'] ) && ! empty( $datas['percent'] ) ) {
	        wp_send_json_success( $datas );
	    } else {
	        wp_send_json_error( $datas );
	    }
	}


	public function MPT_check_post_type( $ID, $post, $update ) {

		global $pagenow;

		// Avoid not selected post types
		$main_settings	= wp_parse_args( get_option( 'MPT_plugin_main_settings' ), $this->MPT_default_options_main_settings( FALSE ) );
		if( !in_array( get_post_type( $ID ), $main_settings['choosed_save_post_post_type'] ) ) {
			return false;
		}

		//$active_posts_types	= wp_parse_args( get_option( 'MPT_plugin_posts_settings' ), $this->MPT_default_options_posts_settings( FALSE ) );
		$active_posts_types = $this->MPT_default_posts_types();

		// Avoid generation when click "Add New"
		if (
			( $pagenow == 'index.php' || $pagenow == 'post.php' ) && 
			in_array( get_post_type( $ID ), $active_posts_types['choosed_post_type'] )
		) {

			$main_settings = get_option( 'MPT_plugin_main_settings' );
			if( isset( $main_settings['rewrite_featured'] ) && ( $main_settings['rewrite_featured'] == true ) )  {
				$rewrite_featured = true;
			} else {
				$rewrite_featured = false;
			}

			$this->MPT_create_thumb( $ID, '0', '1', '0', $rewrite_featured );
		} 
		
	}
	

	/**
	 * Retrieve Image from Database, save it into Media Library, and attach it to the post as featured image
	 *
	 * @since    4.0.0
	*/
	public function MPT_create_thumb( 
		$id, 
		$check_value_enable = 0, 
		$check_post_type = 1, 
		$check_category = 1, 
		$rewrite_featured = 0, 
		$get_only_thumb = false, 
		$extracted_search_term = null,
		$api_chosen = null
	) {

		// Launch logs
		$log = $this->MPT_monolog_call();
		$log->info( 'New generation starting', array( 'post' => $id ) );

		//Avoid revision post type
		if( 'revision' == get_post_type( $id ) ) {
			$log->error( 'This post is a revision. Avoided.', array( 'post' => $id ) );
			return false;
		}

		// Settings
		$main_settings = get_option( 'MPT_plugin_main_settings' );

		// Image location
		$image_location  = ( ! empty( $main_settings['image_location'] ) ) ? $main_settings['image_location']  : 'featured';

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				if( "featured" !== $image_location ) {
					// Check if image generation into content is already done
					$content_post = get_post( $id );
					$image_generated = strpos( $content_post->post_content, 'mpt-img' ) ? true : false;
					if( $image_generated ) {
						return false;
					}
				}

			}
		}

		// Check if thumbnail already exists
		if( has_post_thumbnail( $id ) && ( $rewrite_featured == false ) && ( "featured" === $image_location ) ) {
      		$log->error( 'Featured image already exists', array( 'post' => $id ) );
			return false;
		}

		//$posts_settings          = $this->MPT_default_options_posts_settings();
		$main_settings           = $this->MPT_default_options_main_settings();
		$compatibility_settings  = $this->MPT_default_options_compatibility_settings();


		/* Action 'save_post' triggered when deleting posts. Check if post not trashed */
		if ( 'trash' == get_post_status( $id ) ){
			$log->error( 'Post is in the trash', array( 'post' => $id ) );
			return false;
		}

		if(
			! current_user_can('mpt_manage') &&
			! class_exists( 'Main_WPeMatico' ) &&
			! class_exists( 'FeedWordPress' ) &&
			! class_exists( 'rssPostImporter' ) &&
			! class_exists( 'CyberSyn_Syndicator' ) &&
			! class_exists( 'wp_automatic' )
		) {
			$log->error( 'The user does not have sufficient rights', array( 'post' => $id ) );
			return false;
		}


		// Choosed Post types
		$post_types_default 	= $this->MPT_default_posts_types();
		$post_type_availables	= $post_types_default['choosed_post_type'];
		$categories_availables	= $post_types_default['choosed_categories'];
		/*
		$post_type_availables  = ( ! empty( $posts_settings['choosed_post_type'] ) )  ? $posts_settings['choosed_post_type']  : '';
		// Choosed Categories
		$categories_availables = ( ! empty( $posts_settings['choosed_categories'] ) ) ? $posts_settings['choosed_categories'] : '';*/
		


		if( ! empty( $post_type_availables ) && $check_post_type ) {
			if( ! in_array( get_post_type( $id ), $post_type_availables ) ) {
				$log->error( 'The post is not in selected post types', array( 'post' => $id ) );
				return false;
      		}
		}

		// Check if category match and is a post
		if( ! empty( $categories_availables ) && $check_category && ( 'post' == get_post_type( $id ) ) ) {
			if( ! in_category( $categories_availables, $id ) ) {
				$log->error( 'The post is not in selected categories', array( 'post' => $id ) );
				return false;
			}
		}

		$options       					= wp_parse_args( get_option( 'MPT_plugin_main_settings' ), $this->MPT_default_options_main_settings( TRUE ) );
		$options_banks 					= wp_parse_args( get_option( 'MPT_plugin_banks_settings' ), $this->MPT_default_options_banks_settings( TRUE ) );
		$options_cron  					= wp_parse_args( get_option( 'MPT_plugin_cron_settings' ), $this->MPT_default_options_cron_settings( TRUE ) );
		$options       					= array_merge( $options, $options_banks, $options_cron );

		if( isset( $api_chosen ) ) {
			$options['api_chosen'] = $api_chosen;
		}
		
		$executeElseBlock = true;

		/* Get the full title/tag or a part of the title */
		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				$executeElseBlock = false;

				$posttags       = get_the_tags( $id );
				$postcategories = get_the_category( $id );

				if( TRUE == $get_only_thumb ) {
					$search          = $extracted_search_term;
				} elseif( $options['based_on'] == 'tags' && false != $posttags ) {

					if( $options['tags'] == 'first_tag' ) {
						$search          = $posttags[0]->name;
					} elseif( $options['tags'] == 'last_tag'  ) {
						$search_tag      = end($posttags);
						$search          = $search_tag->name;
					} else {
						$counter_posttags = count( $posttags )-1;
						$rand_counter     = mt_rand( 0, $counter_posttags );
						$search           = $posttags[$rand_counter]->name;
					}

				} elseif( $options['based_on'] == 'categories' && false != $postcategories ) {

					if( $options['categories'] == 'first_tag' ) {
						$search          = $postcategories[0]->name;
					} elseif( $options['categories'] == 'last_tag'  ) {
						$search_category = end($postcategories);
						$search          = $search_category->name;
					} else {
						$counter_category = count( $postcategories )-1;
						$rand_counter     = mt_rand( 0, $counter_category );
						$search           = $postcategories[$rand_counter]->name;
					}

				} elseif( $options['based_on'] == 'custom_field' ) {

					$search = get_metadata( 'post', $id, $options['custom_field'], true );

				} elseif( $options['based_on'] == 'custom_request' ) {

					$base 					= array( '&nbsp;', '%%Title%%', '%%Category%%', '%%Tag%%' );
					$replacement 		= array( ' ', get_post_field( 'post_title', $id, 'raw' ), $postcategories[0]->name, $posttags[0]->name );
					$custom_request = str_replace( $base, $replacement, $options['custom_request'] );

					$search = sanitize_text_field( $custom_request );

				} elseif( $options['based_on'] == 'text_analyser' ) {
					require_once( dirname( ( __FILE__ ) ) . '/../includes/text-miner/TextMiner.php' );

					$content = get_the_content( '', false, $id );

					$tm = new TextMiner();
					$content = str_replace("&nbsp;", '', $content);
					$tm->addText( wp_strip_all_tags( $content ) );

					$tm->convertToLower = TRUE; // optional
					//$tm->includeLowerNGrams = TRUE;

					$tm->process();//should be called before accessing keywords

					$search = $tm->getTopNGrams(1, false);
					$log->info( 'Extracted search term', array( 'post' => $id, 'search term' => $search ) );
				} elseif( $options['based_on'] == 'openai_extractor' ) {

					$search = get_post_field( 'post_title', $id, 'raw' );

					$api_key 			= $options['openai_extractor_apikey'];
					$number_of_keywords = $options['openai_number_of_keywords'];

					if(!$number_of_keywords)
						$number_of_keywords = '1-2';

					if( ! isset( $api_key  ) ) {
						$log->error( 'API Key for OpenAI not provided', array( 'post' => $id ) );
						return false;
					}

					//OpenAI prompt with title
					if( $number_of_keywords == '3' ) {
						$prompt = 'Extract an unique main keyword of 3 words length. If the title is "How to choose a stroller for dolls?", the main keywords should be "baby doll stroller". If the title is "everything you need to know about the incredible world of ants", the main keywords should be "ants insects knowledge".
	The title is "'.$search.'". Result must ONLY be the exact keyword, WITHOUT anything else: no others words and no quotation.';
					} elseif( $number_of_keywords == '2' ) {
						$prompt = 'Extract an unique main keyword of 2 words length. If the title is "How to choose a stroller for dolls?", the main keywords should be "doll stroller". If the title is "everything you need to know about the incredible world of ants", the main keywords should be "Ants insects".
	The title is "'.$search.'". Result must ONLY be the exact keyword, WITHOUT anything else: no others words and no quotation.';
					} else {
						$prompt = 'Extract an unique main keyword of max 1 word or 2 words length. If the title is "How to choose a stroller for dolls?", the main keywords should be "doll stroller". If the title is "everything you need to know about the incredible world of ants", the main keywords should be "ants".
	The title is "'.$search.'". Result must ONLY be the exact keyword, WITHOUT anything else: no others words and no quotation.';
					}

					// Set up the request payload
					$data_openai = array(
						'redirection'		=> 2,
						'method'			=> 'POST',
						'timeout'			=> 30,
						'headers'			=> array('Content-Type' => 'application/json', 'Authorization' => 'Bearer '.$api_key),
						'body' 				=> json_encode( array( "messages" => array( array( 'role' => 'user', 'content' => $prompt ) ), "max_tokens" => 200, "temperature" 	=> 0.5, "model" => "gpt-3.5-turbo" ) ) // text-davinci-003 // "prompt" => $prompt
					);

					$text_openai 					= @wp_remote_request( 'https://api.openai.com/v1/chat/completions', $data_openai ); //https://api.openai.com/v1/completions
					$response_data_openai = json_decode($text_openai['body'], true);

					if( isset( $response_data_openai['error']  ) ) {
						$log->error( 'Error with OpenAI', array( 'post' => $response_data_openai['error']['message'] ) );
						return false;
					}

					$search = sanitize_text_field( $response_data_openai['choices'][0]['message']['content'] ); //$response_data_openai['choices'][0]['text']

					$log->info( 'Extracted from OpenAI', array( 'post' => $id, 'search term' => $search ) );

				} else {
					$search = get_post_field( 'post_title', $id, 'raw' );
				}

			}
		}
		if ($executeElseBlock) {

			if( TRUE == $get_only_thumb ) {
				$search          = $extracted_search_term;
			} elseif( $options['based_on'] == 'text_analyser' ) {
				require_once( dirname( ( __FILE__ ) ) . '/../includes/text-miner/TextMiner.php' );

				$content = get_the_content( '', false, $id );

				$tm = new TextMiner();
				$content = str_replace("&nbsp;", '', $content);
				$tm->addText( wp_strip_all_tags( $content ) );

				$tm->convertToLower = TRUE; // optional
				//$tm->includeLowerNGrams = TRUE;

				$tm->process();//should be called before accessing keywords

				$search = $tm->getTopNGrams(1, false);
				$log->info( 'Extracted search term', array( 'post' => $id, 'search term' => $search ) );
			} else {
				$search = get_post_field( 'post_title', $id, 'raw' );
			}
		}


		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				// If translation is enabled
				if( isset( $options['translation_EN'] ) && ( true == $options['translation_EN'] ) )  {
					$original_search 		= $search;
					$response 		= @wp_remote_request( 'https://translate.googleapis.com/translate_a/single?client=gtx&sl=auto&tl=en&dt=t&q='.urlencode($search) );

					// Check if the request was successful
					if (is_wp_error($response) || wp_remote_retrieve_response_code($response) != 200) {
						// Fallback mechanism in case of an error
						$translated_alt = fallback_translation($alt); // Implement fallback_translation() function
						$log->error('Using fallback translation due to API error', array('error' => is_wp_error($response) ? $response->get_error_message() : 'API response code: ' . wp_remote_retrieve_response_code($response)));
					} else {
						// Extract translation from a successful response
						$response_translation = json_decode(wp_remote_retrieve_body($response), true);
						$search = $response_translation[0][0][0];
						$log->info( 'Translation enabled', array( 'post' => $id, 'new translated term' => $search ) );
					}
				}

			}
		}

		if( isset( $options['title_selection'] ) && $options['title_selection'] == 'cut_title' && isset( $options['title_length'] ) ) {
			$length_title = (int)$options['title_length']-1;
			$search = preg_replace('/((\w+\W*){' . $length_title . '}(\w+))(.*)/', '${1}', $search );
		}


		if( TRUE == $get_only_thumb ) {

			/* SET ALL PARAMETERS */
			$array_parameters = $this->MPT_Get_Parameters( $options, $search );

			$api_url = $array_parameters['url'];
			unset( $array_parameters['url'] );

			if( ! isset( $api_url  ) ) {
				$log->error( 'API URL not provided', array( 'post' => $id ) );
				return false;
			}

			$result_body = $this->MPT_Generate(
				$options['api_chosen'],
				$api_url,
				$array_parameters,
				$options['selected_image'],
				$get_only_thumb,
				$search
			);

			return $result_body;
		}


		// Check if bank option set with array (version MPT 5) or single result
		if( is_array( $options['api_chosen_auto'] ) ) {

			$last_bank_element = end($options['api_chosen_auto']);

			foreach( $options['api_chosen_auto'] as $bank ) {

				$log->info( 'Search with bank ' . $bank, array( 'post' => $id ) );

				// Reset options according new image bank
				$options['api_chosen'] = $bank;
				$array_parameters = $this->MPT_Get_Parameters( $options, $search );
				$api_url = $array_parameters['url'];


				/* GET THE IMAGE URL */
				list( $url_results, $file_media, $alt_img ) = $this->MPT_Generate(
					$bank,
					$api_url,
					$array_parameters,
					$options['selected_image'],
					false,
					$search
				);

				if( ! isset( $url_results ) || ! isset( $file_media ) ) {
					$log->info( 'No results with ' . $bank, array( 'post' => $id ) );

					// Return false with no results with each image bank
					if ( $bank === $last_bank_element ) {
						$log->info( 'No image found with all banks selected', array( 'post' => $id ) );
						return false;
					}
					
				} else {
					// OK
					break;
				}

			}
		} else {

			/* SET ALL PARAMETERS */
			$array_parameters = $this->MPT_Get_Parameters( $options, $search );

			$api_url = $array_parameters['url'];
			unset( $array_parameters['url'] );

			if( ! isset( $api_url  ) ) {
				$log->error( 'API URL not provided', array( 'post' => $id ) );
				return false;
			}


			/* GET THE IMAGE URL */
			list( $url_results, $file_media, $alt_img ) = $this->MPT_Generate(
				$options['api_chosen'],
				$api_url,
				$array_parameters,
				$options['selected_image'],
				false, 
				$search
			);

			if( ! isset( $url_results ) || ! isset( $file_media ) ) {
			
				if( ! isset( $url_results ) || ! isset( $file_media ) ) {
					$log->error( 'No results', array( 'post' => $id ) );
					return false;
				}
	
			}
		}



    	$compatibility = wp_parse_args( get_option( 'MPT_plugin_compatibility_settings' ), $this->MPT_default_options_compatibility_settings( TRUE ) );

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				// Compatibility with the plugin "Featured Image from URL"
				if (
					( true == $compatibility['enable_FIFU'] ) &&
					is_plugin_active( 'featured-image-from-url/featured-image-from-url.php' ) &&
					'dallev1' != $options['api_chosen']
				) {

					global $url_image;
					$url_image = $url_results;

					// Add image url to "Featured Image from URL" field
					fifu_dev_set_image( $id, $url_image );

					$log->info( 'Featured image attached with FIFU plugin', array( 'post' => $id, 'url_image' => $url_image ) );

					return true; // No need more. Stop the run now
				}

			}
		}


		$path_parts    = pathinfo( $url_results );
		$filename      = $path_parts['basename'];
		$wp_upload_dir = wp_upload_dir();

		/* Get the good file extension */
		$filetype   = array( 'image/png', 'image/jpeg', 'image/gif', 'image/bmp', 'image/vnd.microsoft.icon', 'image/tiff', 'image/svg+xml', 'image/svg+xml', 'image/webp' );
		$extensions = array( 'png', 'jpg', 'gif', 'bmp', 'ico', 'tif', 'svg', 'svgz', 'webp' );
		if( isset( $file_media['headers']['content-type'] ) ) {
			$imgextension = str_replace( $filetype, $extensions, $file_media['headers']['content-type'], $count_extension );
			/* Default type if not found : jpg */
			if( (int)$count_extension == 0 )
				$imgextension =  'jpg';
		} else {
			$imgextension = $path_parts['extension'];
		}

		if ( $this->mpt_freemius()->is__premium_only() ) {
			if ( $this->mpt_freemius()->can_use_premium_code() ) {
				// Set original search for image name & title if translated
				if( isset( $options['translation_EN'] ) && ( true == $options['translation_EN'] ) )  {
					$search = $original_search;
				}
			}
		}

		/* Image filename : title extension */
		$search =  str_replace( '%', '', sanitize_title( $search ) ); // Remove % for non-latin characters

		if( $options['image_filename'] == 'date' ) {
			$current_time = current_time( 'Y-m-d' );
			$filename = wp_unique_filename( $wp_upload_dir['path'], $current_time . '.' . $imgextension );
		} elseif( $options['image_filename'] == 'random' ) {
			$filename_rand = wp_rand( 1, 999999 );
			$filename = wp_unique_filename( $wp_upload_dir['path'], $filename_rand . '.' . $imgextension );
		} else {
			$filename = wp_unique_filename( $wp_upload_dir['path'], $search . '.' . $imgextension );
		}

		$folder = $wp_upload_dir['path'] .'/'. $filename;

		if( $file_media['response']['code'] != '200' || empty( $file_media['body'] ) ) {
        $log->error( 'Problem with scrapping', array( 'post' => $id ) );
        return false;
    }

		if ( $file_media['body'] ) {

			/* Upload the file to wordpress directory */
			$file_upload = file_put_contents( $folder, $file_media['body'] );

			/* Convert png to jpeg for dalle */
			$png_jpg    = ( ! empty( $options['dallev1']['convert_jpg'] ) )    ? $options['dallev1']['convert_jpg'] : '' ;
			if( ( true == $png_jpg ) && ( 'dallev1' == $options['api_chosen'] ) ) {
				$image 		= imagecreatefrompng($folder);
				// Remove old png file
				unlink($folder);
				$folder   = str_replace(".png", ".jpg", $folder);
				$filename = str_replace(".png", ".jpg", $filename);
				imagejpeg( $image, $folder, 90 );
				imagedestroy($image);
			}

			if ( $this->mpt_freemius()->is__premium_only() ) {

				if ( $this->mpt_freemius()->can_use_premium_code() ) {

					// Shuffle Image
					if(
						( isset( $options['image_flip'] ) && ( true == $options['image_flip'] ) ) ||
						( isset( $options['image_crop'] ) && ( true == $options['image_crop'] ) )
					)  {
						$image_editor		= wp_get_image_editor( $folder );

						if ( ! is_wp_error( $image_editor ) ) {

							// Invert Image
							if( true == $options['image_flip'] ) {
								$image_editor->flip( false, true );

								$log->info( 'Image flipped horizontally', array( 'post' => $id ) );
							}

							// Crop Image
							if( isset( $options['image_crop'] ) && ( true == $options['image_crop'] ) ) {
								$image_size 		= $image_editor->get_size();
								$image_width		= $image_size['width']*0.9;
								$image_height		= $image_size['height']*0.9;
								$image_width_start	= $image_size['width']*0.05;
								$image_height_start	= $image_size['height']*0.05;
								$image_editor->crop( (int)$image_width_start, (int)$image_height_start, (int)$image_width, (int)$image_height );

								$log->info( 'Image cropped by 10%', array( 'post' => $id ) );
							}

							$image_editor->save( $folder );
						}
					}
				}
			}

			if( $file_upload ) {
				$wp_filetype = wp_check_filetype( basename( $filename ), null );

				$wp_upload_dir = wp_upload_dir();
				$attachment = array(
					'guid'           => $wp_upload_dir['url'] .'/'. urlencode( $filename ),
					'post_mime_type' => $wp_filetype['type'],
					'post_title'     => $search,
					'post_content'   => '',
					'post_status'    => 'inherit'
				);
				$attach_id = wp_insert_attachment( $attachment, $wp_upload_dir['path'] .'/'. urlencode( $filename ) );

				// Add alt text for image
				update_post_meta( $attach_id, '_wp_attachment_image_alt', $alt_img );


				/* Fire filter "wp_handle_upload" for plugins like optimizers etc. */
				$img_values = array(
					'file' => $wp_upload_dir['path'] .'/'. urlencode( $filename ),
					'url' => $wp_upload_dir['url'] .'/'. urlencode( $filename ),
					'type' => $wp_filetype['type']
				);
				apply_filters( 'wp_handle_upload', $img_values );

				require_once( ABSPATH . 'wp-admin/includes/image.php' );
				$attach_data = wp_generate_attachment_metadata( $attach_id, $wp_upload_dir['path'] .'/'.  urlencode( $filename ) );
				$var =  wp_update_attachment_metadata( $attach_id, $attach_data );


				$executeElseBlock = true;

				if ( $this->mpt_freemius()->is__premium_only() ) {

					if ( $this->mpt_freemius()->can_use_premium_code() ) {

						$executeElseBlock = false;

						if( "post_top" === $image_location ) {
							$image_test = wp_get_attachment_image_src( $attach_id, 'large' );
							$alt_text 	= get_post_meta($attach_id, '_wp_attachment_image_alt', true); // Get image 'alt' content
							$content = '<!-- wp:image {"id":'.$attach_id.',"sizeSlug":"large"} -->';
							$content = '<figure class="wp-block-image size-large">';
							$content .= '<img class="mpt-img wp-image-'.$attach_id.'" src="'.$image_test[0].'" alt="' . esc_attr($alt_text) . '" />';
							$content .= '</figure>';
							//$content .= '<!-- /wp:image -->';
							$content = $content . $content_post->post_content;
						} elseif( "post_end" === $image_location ) {
							$image_test = wp_get_attachment_image_src( $attach_id, 'large' );
							$alt_text 	= get_post_meta($attach_id, '_wp_attachment_image_alt', true); // Get image 'alt' content
							$content = '<!-- wp:image {"id":'.$attach_id.',"sizeSlug":"large"} -->';
							$content = '<figure class="wp-block-image size-large">';
							$content .= '<img class="mpt-img wp-image-'.$attach_id.'" src="'.$image_test[0].'" alt="' . esc_attr($alt_text) . '" />';
							$content .= '</figure>';
							//$content .= '<!-- /wp:image -->';
							$content = $content_post->post_content . $content;
						} else {
							$log->info( 'Featured image added', array( 'post' => $id, 'image' => $attach_id ) );
							set_post_thumbnail( $id, $attach_id );
						}
					}
				}
				if ($executeElseBlock) {
					$log->info( 'Featured image added', array( 'post' => $id, 'image' => $attach_id ) );
					set_post_thumbnail( $id, $attach_id );
				}

				// Link the media to the post
				$media_link_uploaded_to = wp_update_post( array(
					'ID'            => $attach_id,
					'post_parent'   => $id,
				), true );


				if ( $this->mpt_freemius()->is__premium_only() ) {
					
					if ( $this->mpt_freemius()->can_use_premium_code() ) {

						if( "featured" !== $image_location ) {

							// Array of post category ids
							$arr_post_category = array();
							foreach( $postcategories as $postcategory ) {
								$arr_post_category[] = (int)$postcategory->cat_ID;
							}

							/*
							* Remove save_post Action to avoid infinite loop.
							* cf https://developer.wordpress.org/reference/hooks/save_post/#avoiding-infinite-loops
							* Use "remove_all_actions" instead of "remove_action". Otherwise the log file bug
							* remove_action( 'save_post', array( &$this, 'MPT_create_thumb' ) );
							*/
							remove_all_actions( 'save_post' );

							wp_insert_post(
								array(
									'ID'                    => $id,
									'post_content'          => $content,
									'post_category'         => $arr_post_category,
									'post_title'            => $content_post->post_title,
									'post_status'           => $content_post->post_status,
									'post_author'           => $content_post->post_author,
									'post_date'             => $content_post->post_date,
									'post_date_gmt'         => $content_post->post_date_gmt,
									'post_excerpt'          => $content_post->post_excerpt,
									'comment_status'        => $content_post->comment_status,
									'ping_status'           => $content_post->ping_status,
									'post_password'         => $content_post->post_password,
									'post_name'             => $content_post->post_name,
									'to_ping'               => $content_post->to_ping,
									'pinged'                => $content_post->pinged,
									'post_modified'         => $content_post->post_modified,
									'post_modified_gmt'     => $content_post->post_modified_gmt,
									'post_content_filtered' => $content_post->post_content_filtered,
									'post_parent'           => $content_post->post_parent,
									'menu_order'            => $content_post->menu_order,
									'post_type'             => $content_post->post_type,
									'post_mime_type'        => $content_post->post_mime_type
								)
							);


							add_action( 'save_post', array( &$this, 'MPT_create_thumb' ) );
							$log->info( 'Image added into the post', array( 'post' => $id, 'image' => $attach_id ) );
						}
					}
				}

        		do_action( 'mpt_after_create_thumb', $id, $attach_id );

				return $attach_id;
			}
		}

	}



	/**
	 * Get all settings from user
	 *
	 * @since    4.0.0
	*/
  private function MPT_Get_Parameters( $options, $search ) {

	/* GOOGLE IMAGE SCRAPING PARAMETERS */
	if( $options['api_chosen'] == 'google_scraping' ) {

		$country   = ( ! empty( $options['google_scraping']['search_country'] ) )	? $options['google_scraping']['search_country'] : 'en' ;
		$img_color = ( ! empty( $options['google_scraping']['img_color'] ) )		? $options['google_scraping']['img_color'] : '' ;
		$imgsz     = ( ! empty( $options['google_scraping']['imgsz'] ) )			? $options['google_scraping']['imgsz'] : '' ;
		$format    = ( ! empty( $options['google_scraping']['format'] ) )			? $options['google_scraping']['format'] : '' ;
		$imgtype   = ( ! empty( $options['google_scraping']['imgtype'] ) )			? $options['google_scraping']['imgtype'] : '' ;
		$rights    = ( ! empty( $options['google_scraping']['rights'] ) )			? $options['google_scraping']['rights'] : '' ;
		$safe      = ( ! empty( $options['google_scraping']['safe'] ) )				? $options['google_scraping']['safe'] : 'medium' ;

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				// Domains Restricted
				$domains = $options['google_scraping']['restricted_domains'];
				if( ! empty( $domains ) ) {
					$array_domains = explode("\n", $domains);

					foreach( $array_domains as $key => $domain  ){
						reset( $array_domains );
						if ( $key === key( $array_domains ) ) {
							$search_domains = " site:$domain";
						} else {
							$search_domains .= " OR site:$domain";
						}
					}

					$search = $search . $search_domains;
				}

				// Domains Blacklisted
				$blacklisted_domains = $options['google_scraping']['blacklisted_domains'];
				if( ! empty( $blacklisted_domains ) ) {
					$array_blacklisted_domains = explode("\n", $blacklisted_domains);

					foreach( $array_blacklisted_domains as $key_blacklisted_domains => $blacklisted_domains  ){
						reset( $array_blacklisted_domains );

												if ( $key_blacklisted_domains === key( $array_blacklisted_domains ) ) {
							$search_blacklisted_domains = " -site:$blacklisted_domains";
						} else {
							$search_blacklisted_domains .= " -site:$blacklisted_domains";
						}
					}

					$search = $search . $search_blacklisted_domains;
				}
			}
		}

		// Old API option. Replace value for safe
		if( $safe == 'moderate' ) {
			$safe = 'medium';
		}

		// Remove very special characters
		$search = str_replace('…', '', $search);

		$array_parameters = array(
			'url'               => 'http://www.google.com/search',
			'tbm'               => 'isch',
			'q'                 => urlencode( $search ),
			'hl'                => $country,
			'safe'              => $safe,
			'rsz'               => '3',
			'tbs'               => '',
		);

		if( ! empty( $rights ) ) {
			$array_parameters['tbs'] .= 'sur:' . $rights . ',';
		}

		if( ! empty( $imgtype ) ) {
			$array_parameters['tbs'] .= 'itp:' . $imgtype . ',';
		}

		if( ! empty( $imgsz ) ) {
			$array_parameters['tbs'] .= 'isz:' . $imgsz . ',';
		}

		if( ! empty( $format ) ) {
			$array_parameters['tbs'] .= 'iar:' . $format . ',';
		}

		if( ! empty( $img_color ) ) {
			$array_parameters['tbs'] .= 'ic:specific,isc:' . $img_color;
		}

	}
	/* GOOGLE IMAGE API PARAMETERS */
	elseif( $options['api_chosen'] == 'google_image' ) {

		if( empty( $options['googleimage']['cxid'] ) || empty( $options['googleimage']['apikey'] ) ) {
			return false;
		}

		$country   = ( !empty( $options['googleimage']['search_country'] ) )? $options['googleimage']['search_country'] : 'en' ;
		$img_color = ( !empty( $options['googleimage']['img_color'] ) )     ? $options['googleimage']['img_color'] : '' ;
		$filetype  = ( !empty( $options['googleimage']['filetype'] ) )      ? $options['googleimage']['filetype'] : '' ;
		$imgsz     = ( !empty( $options['googleimage']['imgsz'] ) )         ? $options['googleimage']['imgsz'] : 'large' ;
		$imgtype   = ( !empty( $options['googleimage']['imgtype'] ) )       ? $options['googleimage']['imgtype'] : '' ;
		$safe      = ( !empty( $options['googleimage']['safe'] ) )          ? $options['googleimage']['safe'] : 'medium' ;
		// Old API option. Replace value for safe
		if( $safe == 'moderate' ) {
			$safe = 'medium';
		}

		if( isset( $options['googleimage']['rights'] ) && ! empty( $options['googleimage']['rights'] ) ) {
			$rights = '(';
			$last_right = array_keys( $options['googleimage']['rights'] );
			$last_right = end( $last_right );
			foreach( $options['googleimage']['rights'] as $rights_into_searching ) {
				$rights .= $rights_into_searching;
				if ( $rights_into_searching != $last_right ) {
					$rights .= '|';
				}
			}
			$rights .= ')';
		} else {
			$rights = '';
		}

		$array_parameters = array(
			'url'               => 'https://www.googleapis.com/customsearch/v1',
			'imgSize'           => $imgsz,
			'rights'            => $rights,
			'imgtype'           => $imgtype,
			'hl'                => $country,
			'filetype'          => $filetype,
			'safe'              => $safe,
			'rsz'               => '3',
			'q'                 => urlencode( $search ),
			'userip'            => $_SERVER['SERVER_ADDR'],
			'cx'                => trim( $options['googleimage']['cxid'] ),
			'key'               => trim( $options['googleimage']['apikey'] )
		);

		if( ! empty( $img_color ) ) {
			$array_parameters['imgDominantColor'] = $img_color;
		}

	}
	/* DALL-E API v3 PARAMETERS */
	elseif( $options['api_chosen'] == 'dallev1' ) {

		$api_key          = ( ! empty( $options['dallev1']['apikey'] ) )         ? $options['dallev1']['apikey'] : '' ;
		$img_size         = ( ! empty( $options['dallev1']['imgsize'] ) )        ? $options['dallev1']['imgsize'] : '1024x1024' ;

		$array_parameters = array(
			'url'           	=> 'https://api.openai.com/v1/images/generations',
			'redirection'       => 2,
			'method'            => 'POST',
			'timeout'			=> 30,
			'headers'			=> array('Content-Type' => 'application/json', 'Authorization' => 'Bearer '.$api_key),
			'body' 				=> json_encode( array( "model" => "dall-e-3", "prompt" => $search, "n" => 1, "size" => $img_size ) )
		);

	}
	/* FLICKR PARAMETERS */
	elseif( $options['api_chosen'] == 'flickr' ) {

		$api_key = '63d9c292b9e2dfacd3a73908779d6d6f';
		$imgtype =( ! empty( $options['flickr']['imgtype']) )? $options['flickr']['imgtype'] : '7' ;
		if( isset( $options['flickr']['rights'] ) && ! empty( $options['flickr']['rights'] ) ) {
			$rights = '';
			$last_right = array_keys( $options['flickr']['rights'] );
			$last_right = end( $last_right );
			foreach( $options['flickr']['rights'] as $rights_into_searching ) {
				$rights .= $rights_into_searching;
				if ( $rights_into_searching != $last_right ) {
					$rights .= ',';
				}
			}
		} else {
			$rights = '0,1,2,3,4,5,6,7,8';
		}

		$array_parameters = array(
			'url'               => 'https://api.flickr.com/services/rest/',
			'method'            => 'flickr.photos.search',
			'api_key'           => $api_key,
			'text'              => urlencode( $search ),
			'per_page'          => '16',
			'format'            => 'json',
			'nojsoncallback'    => '1',
			'privacy_filter'    => '1',
			'license'           => $rights,
			'sort'              => 'relevance',
			'content_type'      => $imgtype,
		);

	}
	/* PIXABAY PARAMETERS */
	elseif( $options['api_chosen'] == 'pixabay' ) {

		$pixabay_username = ( ! empty( $options['pixabay']['username'] ) )       ? $options['pixabay']['username'] : '' ;
		$api_key          = ( ! empty( $options['pixabay']['apikey'] ) )         ? $options['pixabay']['apikey'] : '' ;
		$imgtype          = ( ! empty( $options['pixabay']['imgtype'] ) )        ? $options['pixabay']['imgtype'] : 'all' ;
		$country          = ( ! empty( $options['pixabay']['search_country'] ) ) ? $options['pixabay']['search_country'] : 'en' ;
		$orientation      = ( ! empty( $options['pixabay']['orientation'] ) )    ? $options['pixabay']['orientation'] : 'all' ;
		$safe             = ( ! empty( $options['pixabay']['safesearch'] ) )     ? $options['pixabay']['safesearch'] : 'false' ;
		$min_width        = ( ! empty( $options['pixabay']['min_width'] ) )      ? (int)$options['pixabay']['min_width'] : '0' ;
		$min_height       = ( ! empty( $options['pixabay']['min_height'] ) )     ? (int)$options['pixabay']['min_height'] : '0' ;

		$array_parameters = array(
			'url'           => 'https://pixabay.com/api/',
			'username'      => $pixabay_username,
			'key'           => $api_key,
			'lang'          => $country,
			'q'             => urlencode( $search ),
			'image_type'    => $imgtype,
			'per_page'      => '200',
			'orientation'   => $orientation,
			'safesearch'    => $safe,
			'min_width'     => $min_width,
			'min_height'    => $min_height,
		);

	}

	/* YOUTUBE PARAMETERS */
	elseif( $options['api_chosen'] == 'youtube' ) {

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				$api_key          = ( ! empty( $options['youtube']['apikey'] ) ) ? $options['youtube']['apikey'] : '' ;

				$array_parameters = array(
					'url'           => 'https://www.googleapis.com/youtube/v3/search',
					'key'           => $api_key,
					'part'          => 'snippet',
					'type'          => 'video',
					'maxResults'	=> 50,
					'q'             => urlencode( $search ),
				);

			}

		}
	}

	/* PEXELS PARAMETERS */
	elseif( $options['api_chosen'] == 'pexels' ) {

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				$api_key          = ( ! empty( $options['pexels']['apikey'] ) ) ? $options['pexels']['apikey'] : '' ;
				$search_country   = ( ! empty( $options['pexels']['search_country'] ) ) ? $options['pexels']['search_country'] : 'en-US' ;
				$orientation      = ( ! empty( $options['pexels']['orientation'] ) ) ? $options['pexels']['orientation'] : '' ;
				$size      				= ( ! empty( $options['pexels']['size'] ) ) ? $options['pexels']['size'] : 'small' ;


				$array_parameters = array(
					'url'		=> 'https://api.pexels.com/v1/search?query=' . urlencode( $search ) . '&per_page=80&locale=' . $search_country . '&orientation=' . $orientation . '&size=' . $size,
					'headers'	=> array( 'Authorization' => $api_key ),
				);

			}

		}
	}

	/* ENVATO ELEMENTS PARAMETERS */
	elseif( $options['api_chosen'] == 'envato' ) {

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				$orientation		= ( ! empty( $options['envato']['orientation'] ) ) ? $options['envato']['orientation'] : '' ;
				$number_of_people	= ( ! empty( $options['envato']['number_of_people'] ) ) ? urlencode($options['envato']['number_of_people']) : '' ;
				$envato_token		= ( ! empty( $options['envato']['envato_token'] ) ) ? $options['envato']['envato_token'] : '' ;

				$array_parameters = array(
					'url'           		=> 'https://api.extensions.envato.com/extensions/search',
					'type'					=> 'photos',
					'orientation'   		=> $orientation,
					'number_of_people'		=> $number_of_people,
					'envato_token'  		=> $envato_token,
					'search_terms'  		=> urlencode( $search ),
				);
			
			}
		}
	}

	/* UNSPLASH PARAMETERS */
	elseif( $options['api_chosen'] == 'unsplash' ) {

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				$client_id 		= ( ! empty( $options['unsplash']['apikey'] ) ) ? $options['unsplash']['apikey'] : '' ;
				$search_country = ( ! empty( $options['unsplash']['search_country'] ) )  ? $options['unsplash']['search_country'] : 'en' ;

				$search = str_replace("’", "'", $search);

				$array_parameters = array(
					'url'  => 'https://api.unsplash.com/search/photos?client_id=' . $client_id . '&query=' . urlencode( strtolower( $search ) ) . '&per_page=100&lang=' . $search_country,
				);
			}
		}
	}

  /* CC SEARCH PARAMETERS */
	elseif( $options['api_chosen'] == 'cc_search' || $options['api_chosen'] == 'openverse' ) {

	  $imgtype        = ( ! empty( $options['cc_search']['imgtype'] ) )       ? $options['cc_search']['imgtype'] : '' ;
	  $aspect_ratio   = ( ! empty( $options['cc_search']['aspect_ratio'] ) )  ? $options['cc_search']['aspect_ratio'] : '' ;

	  $sources = array(
		  'wordpress',
	      'woc_tech',
	      'wikimedia',
	      'wellcome_collection',
	      'thorvaldsensmuseum',
	      'thingiverse',
	      'svgsilh',
	      'statensmuseum',
	      'spacex',
	      'smithsonian_zoo_and_conservation',
	      'smithsonian_postal_museum',
	      'smithsonian_portrait_gallery',
	      'smithsonian_national_museum_of_natural_history',
	      'smithsonian_libraries',
	      'smithsonian_institution_archives',
	      'smithsonian_hirshhorn_museum',
	      'smithsonian_gardens',
	      'smithsonian_freer_gallery_of_art',
	      'smithsonian_cooper_hewitt_museum',
	      'smithsonian_anacostia_museum',
	      'smithsonian_american_indian_museum',
	      'smithsonian_american_history_museum',
	      'smithsonian_american_art_museum',
	      'smithsonian_air_and_space_museum',
	      'smithsonian_african_art_museum',
	      'smithsonian_african_american_history_museum',
	      'sketchfab',
	      'sciencemuseum',
	      'rijksmuseum',
	      'rawpixel',
	      'phylopic',
	      'nypl',
	      'nasa',
	      'museumsvictoria',
	      'met',
	      'mccordmuseum',
	      'iha',
	      'geographorguk',
	      'floraon',
	      'flickr',
	      'europeana',
	      'eol',
	      'digitaltmuseum',
	      'deviantart',
	      'clevelandmuseum',
	      'brooklynmuseum',
	      'bio_diversity',
	      'behance',
	      'animaldiversity',
	      'WoRMS',
	      'CAPL',
	      '500px',
	  );

	  $sources_with_comma = implode( ",", $sources );

	  $array_parameters = array(
	          'url'				=> 'https://api.openverse.engineering/v1/images/',//'https://api.creativecommons.engineering/v1/images',
	          'q'				=> urlencode( $search ),
	          'imgtype'			=> urlencode( $imgtype ),
	          'aspect_ratio'	=> urlencode( $aspect_ratio ),
	          'source'			=> $sources_with_comma
	  );
	}

	else {
		return false;
	}

	return $array_parameters;

  }


	/**
	 * Generate image process
	 *
	 * @since    4.0.0
	*/
  public function MPT_Generate( $service, $url, $url_parameters, $selected_image, $get_only_thumb = false, $search = '' ) {

	$log = $this->MPT_monolog_call();

	list( $result_body, $result ) = $this->MPT_get_results( $service, $url, $url_parameters );

	// In case of API problem
	if( empty( $result_body ) ) {
		return false;
	}


	if( $service == 'google_image' ) {
		$loop_results = $result_body['items'];
		// TODO : Check if urls are real images or just redirections
		$url_path     = 'pagemap';
	} elseif( $service == 'google_scraping' ) {
		$loop_results = $result_body['results'];
		$url_path     = 'url';
	} elseif( $service == 'dallev1' ) {
		$loop_results = $result_body['data'];
		$url_path     = 'url';
	} elseif( $service == 'flickr' ) {
		$loop_results = $result_body['photos']['photo'];
		$url_path     = 'id';
	} elseif( $service == 'pixabay' ) {
		$loop_results = $result_body['hits'];
		$url_path     = 'largeImageURL';
	} elseif( $service == 'youtube' ) {
		$loop_results = $result_body['items'];
		$url_path     = 'id';
	} elseif( $service == 'pexels' ) {
		$loop_results = $result_body['photos'];
		$url_path     = 'src';
	} elseif( $service == 'unsplash' ) {
		$loop_results = $result_body['results'];
		$url_path     = 'urls';
	} elseif( $service == 'cc_search' ) {
		$loop_results = $result_body['results'];
		$url_path     = 'url';
	} elseif( $service == 'envato' ) {
		$loop_results = $result_body['results']['search_query_result']['search_payload']['items'];
		$url_path     = 'humane_id';
	} else {
		return false;
	}

	// Check if function is launch for Gutnberg block
	if( ( TRUE == $get_only_thumb ) && ( $service == 'envato' ) ) {
		return $result_body['results']['search_query_result']['search_payload'];
	} else if( ( TRUE == $get_only_thumb ) && ( 'flickr' != $service ) ) {
		return $result_body;
	} else {}


	if ( $this->mpt_freemius()->is__premium_only() ) {
		if ( $this->mpt_freemius()->can_use_premium_code() ) {
			/* Random Image */
			if( ( $selected_image == 'random_result' ) ) {
				@shuffle( $loop_results );
			}
		}
	}

	// Testing images
	if( $service == 'google_scraping' ) {
		foreach ( $loop_results as $loop_result_result => $loop_result ) {

			$remote_img      = wp_remote_head( $loop_result['url'] );
			$remote_response = wp_remote_retrieve_response_code( $remote_img );

			$log = $this->MPT_monolog_call();
			$log->info( 'remote_img', array( 'remote_img' => $loop_result ) );

			if( 200 !== $remote_response ) {
				// Remove the result, image not valid
				unset( $loop_results[$loop_result_result] );
			} else {
				// Image ok. Avoid next results.
				break;
			}

			$infos_img = @getimagesize( $loop_result['url'] );

			if( false === $infos_img ) {
				// Remove the result, image not valid
				unset( $loop_results[$loop_result_result] );
			} else {
				// Image ok. Avoid next results.
				break;
			}
		}
	}





	if( !empty( $loop_results ) ) {
		$loop_count = 0;
		$numUrl     = count( $loop_results );
		foreach( $loop_results as $fetch_result ) {

			$url_result = $fetch_result[$url_path];

			// Change default url image
			if( $service == 'google_image' ) {
					$url_result = $url_result['cse_image'][0]['src'];
			} elseif( $service == 'unsplash' ) {
					$url_result = $url_result['full'];
			} elseif( $service == 'pexels' ) {
					$url_result = $url_result['original'];
			} elseif( $service == 'dallev1' ) {
					// Show revised prompt (by openAI) in logs
					$log = $this->MPT_monolog_call();
					$log->info( 'DALL_E',  array( 'revised_prompt' => $fetch_result['revised_prompt'] ) );
			} else {
					$url_result = $fetch_result[$url_path];
			}


			$options = wp_parse_args( get_option( 'MPT_plugin_main_settings' ), $this->MPT_default_options_main_settings( TRUE ) );

			if ( $this->mpt_freemius()->is__premium_only() ) {

				if ( $this->mpt_freemius()->can_use_premium_code() ) {

					// Alt texts
					if( 
						isset( $options['enable_alt'] ) && 
						( 'enable' == $options['enable_alt'] )
					) {
						if( 'based_on' == $options['alt_from'] ) {
								$alt 		= $search;
						} elseif( $service == 'google_scraping' ) { 
								$alt 		= $fetch_result['alt'];
						} elseif( $service == 'unsplash' ) {
								$alt = $fetch_result['description'];
								if( empty( $alt ) ) {
									$alt = $fetch_result['alt_description'];
								}
						} elseif( $service == 'pexels' ) {
								$alt 		= $fetch_result['alt'];
						} elseif( $service == 'pixabay' ) {
								$keywords 	= explode(", ", $fetch_result['tags'] );
								$alt 		= $keywords[0];
						} elseif( $service == 'flickr' ) {
								$alt 		= $fetch_result['title'];
						} elseif( $service == 'cc_search' ) {
								$alt 		= $fetch_result['title'];
						} elseif( $service == 'youtube' ) {
								$alt 		= $fetch_result['snippet']['title'];
						} elseif( $service == 'envato' ) {
								$alt 		= $fetch_result['description'];
						} else {
								$alt 		= '';
						}

						$log->info( 'Alt image enabled', array( 'alt' => $alt ) );

						// If alt translation is enabled
						if( 
							isset( $options['translate_alt'] ) && 
							( true == $options['translate_alt'] )
						)  {

							if( !isset( $options['translate_alt_lang'] ) ) {
								$wp_lang    = get_bloginfo('language');
								$alt_lang   = substr( $wp_lang, 0, 2 );
							} else {
								$alt_lang   = $options['translate_alt_lang'];
							}

							// Sending a request to the Google Translate API
							$response = @wp_remote_request('https://translate.googleapis.com/translate_a/single?client=gtx&sl=en&tl=' . $alt_lang . '&dt=t&q=' . urlencode($alt));

							// Check if the request was successful
							if (is_wp_error($response) || wp_remote_retrieve_response_code($response) != 200) {
								// Fallback mechanism in case of an error
								$translated_alt = fallback_translation($alt); // Implement fallback_translation() function
								$log->error('Using fallback translation due to API error', array('error' => is_wp_error($response) ? $response->get_error_message() : 'API response code: ' . wp_remote_retrieve_response_code($response)));
							} else {
								// Extract translation from a successful response
								$response_translation = json_decode(wp_remote_retrieve_body($response), true);
								$translated_alt = $response_translation[0][0][0];
							}

							// Logging the translation operation
							$log->info('Alt Image Translation enabled', array('translated_alt' => $translated_alt));
							$alt = $translated_alt;
						}

					}
				}
			} else {
				$alt = '';
			}


			// FLICKR : Additional remote request to get image url
			if( $service == 'flickr' ) {
					$api_key                = '63d9c292b9e2dfacd3a73908779d6d6f';
					$url                    = 'https://api.flickr.com/services/rest/?method=flickr.photos.getSizes&api_key=' . $api_key . '&photo_id=' . $url_result . '&format=json&nojsoncallback=1';
					$result_img_flickr      = wp_remote_request( $url );
					$result_img_body_flickr = json_decode( $result_img_flickr['body'], true );
					$result                 = end( $result_img_body_flickr['sizes']['size'] );
					$url_result             = $result['source'];
					//$url_result_sizes       = $result_img_body_flickr['sizes'];
			}

			// ENVATO : Additional remote request to get image url
			if( $service == 'envato' ) {

					$url 				= 'https://api.extensions.envato.com/extensions/item/' . $url_result . '/download';
					$project_ags 		= array( 'project_name' => get_bloginfo('name') );
					$result_img_envato 	= wp_remote_post(
						add_query_arg($project_ags, $url),
						array(
							'headers' => array(
								"Extensions-Extension-Id" 	=> md5( get_site_url() ),
								"Extensions-Token" 			=> $url_parameters['envato_token'],
								"Content-Type"				=> "application/json"
							),
						)
					);
					$result 			= json_decode( $result_img_envato['body'] );
					$url_result			 = $result->download_urls->max2000;

			}

			// YOUTUBE : Additional remote request to get thumbnail
			if( $service == 'youtube' ) {
					$api_key                = $url_parameters['key'];
					$url                    = 'https://www.googleapis.com/youtube/v3/videos?key=' . $api_key . '&part=snippet&id=' . $fetch_result['id']['videoId'];
					$result_img_yt          = wp_remote_request( $url );
					$result_img_body_yt     = json_decode( $result_img_yt['body'], true );
					$hdimg                  = end( $result_img_body_yt['items'][0]['snippet']['thumbnails'] );
					$url_result             = $hdimg['url'];
			}

			if( empty( $url_result ) )
				continue;

			// Avoid unknown image type
			if( ( $service != 'dallev1' ) && ( $service != 'unsplash' ) && ( $service != 'pexels' ) && ( $service != 'envato' ) ) {
				$url_result 	= $url_result;
				$wp_filetype 	= wp_check_filetype( $url_result );

				if( false == $wp_filetype['type'] )
					continue;
			}


			if( TRUE == $get_only_thumb ) {
				$url_result_ar['photos'][]['url'] = $url_result;
			} else {
				$file_media = @wp_remote_request( $url_result );
				if( isset( $file_media->errors ) || $file_media['response']['code'] != 200 || strpos( $file_media['headers']['content-type'], 'text/html' ) !== false ) {
					if( ++$loop_count === $numUrl )
						return false;
					else
						continue;
				} else {
					break;
				}
			}
			
		}

		if( TRUE == $get_only_thumb ) {
			return $url_result_ar;
		}

	} else {
		return false;
	}

	return array( $url_result, $file_media, $alt );

}




	/**
	 * Image results to get thumbnails
	 *
	 * @since    4.0.0
	*/
	private function MPT_get_results( $service, $url, $url_parameters ) {

		if( $service == 'dallev1' || $service == 'pexels' ) {
			$defaults = $url_parameters;
		} else {
			/* Retrieve 3 images as result */
			$url         = add_query_arg( $url_parameters, $url );
			// Simulate Default Browser
			$defaults    = array(
			'redirection'        => 9,
			'user-agent'         => 'Mozilla/5.0 (Linux; Android 6.0.1; Nexus 5X Build/MMB29P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.96',
			'reject_unsafe_urls' => false,
			'sslverify'          => false,
			);
		}


		// Proxy settins
		$options_proxy = get_option( 'MPT_plugin_proxy_settings' );

		if ( $this->mpt_freemius()->is__premium_only() ) {

			if ( $this->mpt_freemius()->can_use_premium_code() ) {

				if( is_array( $options_proxy ) ) {
					if (
						( 'enable' == $options_proxy['enable_proxy'] ) &&
						( ! empty( $options_proxy['proxy_adress'] ) ) &&
						( ! empty( $options_proxy['proxy_port'] ) )
					) {
						// Proxy settings
						define( 'WP_PROXY_HOST', $options_proxy['proxy_adress'] );
						define( 'WP_PROXY_PORT', $options_proxy['proxy_port'] );

						if( $options_proxy['proxy_username'] ) {
							define( 'WP_PROXY_USERNAME', $options_proxy['proxy_username'] );
						}

						if( $options_proxy['proxy_password'] ) {
							define( 'WP_PROXY_PASSWORD', $options_proxy['proxy_password'] );
						}
					}
				}
			}
		}

		$result      = wp_remote_request( $url, $defaults );

		// If error happen
		if( ! empty( $result->errors['http_request_failed'] ) ) {
			return false;
		}

		// Google Scraping : Different method
		if( $service == 'google_scraping' ) {


			// Get all alts from Google
			preg_match_all( '/data-pt="([^"]*)"/', $result['body'], $output_img_alts );

			// Get all images from Google
			//preg_match_all( '/,\["http[^"]((?!gstatic).)*",\d+?,\d+?\]/', $result['body'], $output_img_urls );
			preg_match_all( '/data-ou="(http[^"]*)"/', $result['body'], $output_img_urls );

			$result_body['results'] = array_map(
				array( &$this, 'MPT_order_array_urls' ),
				$output_img_urls[1],
				$output_img_alts[1]
			);

		} else {
			$result_body = json_decode( $result['body'], true );

			if( $result['response']['code'] != '200' )
				return false;
		}

		return array( $result_body, $result );
	}


	/**
	 * Remove gstatic images
	 *
	 * @since    4.0.0
	*/
	public function MPT_order_array_urls( $str, $str_alt ) {

	  // Get only the url and exclude Google image url (domain gstatic)
	  $pattern 		= '/,\["(http[^"]((?!gstatic).)*)",\d+?,\d+?\]/';
	  $replacement 	= '$1';
	  $real_url 	= preg_replace( $pattern, $replacement, $str );

	  return array( 
			'url' => $real_url, 
			'alt' => $str_alt 
		);
	}

}
