<?php
if ( ! function_exists( 'add_filter' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
		exit();
}

// unschedule event upon plugin deactivation
function crons_deactivate() {
	// find out when the last event was scheduled
	$timestamp = wp_next_scheduled ('mpt_scheduled_cron');
	// unschedule previous event if any
	wp_unschedule_event ($timestamp, 'mpt_scheduled_cron');
}

register_deactivation_hook (__FILE__, 'crons_deactivate');


// create a scheduled event (if it does not exist already)
function crons_activation() {

	if( !wp_next_scheduled( 'mpt_scheduled_cron' ) ) {
		wp_schedule_event( time(), 'mptcronpro', 'mpt_scheduled_cron' );
	}

}
// and make sure it's called whenever WordPress loads
add_action('init', 'crons_activation');



add_filter( 'cron_schedules', 'crons_scheduled' );
// add another interval
function crons_scheduled( $schedules ) {

	$options  = wp_parse_args( get_option( 'MPT_plugin_cron_settings' ) );

	// Set interval according word chosen : minutes, hours, days
	$default_timestamp_cron = 600;
	if( isset( $options['cron_word'] ) ) {
		if( 'minutes' === $options['cron_word'] ) {
			$interval = $options['cron_number_minutes'];
		} elseif( 'hours' === $options['cron_word'] ) {
			$interval = $options['cron_number_hours'];
		} elseif( 'days' === $options['cron_word'] ) {
			$interval = $options['cron_number_days'];
		} else {
			$interval = $default_timestamp_cron; // Default 10 minutes
		}
	}
	//  $options['cron_interval'] was until v2.4.10
	elseif( isset( $options['cron_interval'] ) ) {
		$old_interval = absint( $options['cron_interval'] );
		if( $default_timestamp_cron > $old_interval ) {
			$interval = $default_timestamp_cron; // Default 10 minutes
		} else {
			$interval = $old_interval;
		}
	} else {
		$interval = $default_timestamp_cron; // Default 10 minutes
	}

	$schedules['mptcronpro'] = array(
		'interval' => (int)$interval,
		'display'  => esc_html__( 'Magic Post Thumbnail Pro - Cron', 'mpt' )
	);
	return $schedules;
}


// hook that function onto our scheduled event:
add_action( 'mpt_scheduled_cron', 'mpt_add_crons', 10, 1 );

function mpt_add_crons() {

	$options  = wp_parse_args( get_option( 'MPT_plugin_cron_settings' ) );


	if( !empty( $options['number_posts'] ) ) {
		$number_posts = absint( $options['number_posts'] );
	} else  {
		$number_posts = '20'; // 20 posts by default
	}

	// If Posts date is set, choose date
	if( 'interval' === $options['posts_date'] ) {

		$chosen_word = ( 'hours' === $options['cron_interval_word'] )? 'hours': 'days';

		// Hours
		if( 'hours' === $chosen_word ) {
			$unit = absint( $options['posts_date_select_interval_hours'] );

			if( $unit > 23 ) {
				$unit = 23;
			}
		}
		// Days
		else {
			$unit = absint( $options['posts_date_select_interval_days'] );

			if( $unit > 31 ) {
				$unit = 31;
			}
		}

		// Create Query arguments
		$args_cron_posts = array(
			'post_type'      => $options['cron_post_type'],
			'posts_per_page' => $number_posts,
			'orderby'				 => 'ID',
			'order'					 => 'ASC',
			'meta_query' => array(
        array(
         'key' => '_thumbnail_id',
         'compare' => 'NOT EXISTS'
        ),
    	),
			'date_query'     => array(
				array(
					'after' => $unit . ' ' . $chosen_word . ' ago'
				),
			)
		);
	}
	// All posts chosen
	else {
		$args_cron_posts = array(
			'post_type'      => $options['cron_post_type'],
			'posts_per_page' => $number_posts,
			'orderby'				 => 'ID',
			'order'					 => 'ASC', // Start with older posts
			'meta_query' => array(
        array(
         'key' => '_thumbnail_id',
         'compare' => 'NOT EXISTS'
        ),
    	)
		);
	}

	$query_cron_posts = new WP_Query( $args_cron_posts );

	while ( $query_cron_posts->have_posts() ) {
		$query_cron_posts->the_post();

		$launch_MPT     = new Magic_Post_Thumbnail();
    $launch_MPT2    = new Magic_Post_Thumbnail_Generation( $launch_MPT->get_plugin_name(), $launch_MPT->get_version() );
    $MPT_return     = $launch_MPT2->MPT_create_thumb( get_the_ID(), '0', '0', '0', '0' );
	}
}

?>
