<?php

/**
 *
 * @link              https://magic-post-thumbnail.com/
 * @since             1.0.0
 * @package           Magic_Post_Thumbnail
 *
 * @wordpress-plugin
 * Plugin Name: Magic Post Thumbnail (Premium)
 * Plugin URI:        http://wordpress.org/plugins/magic-post-thumbnail/
 * Description:       Transform your posts with stunning images effortlessly! Magic Post Thumbnail includes many image banks to automatically get featured images for your posts.
 * Version:           5.1.1
 * Update URI: https://api.freemius.com
 * Author:            Magic Post Thumbnail
 * Author URI:        https://magic-post-thumbnail.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       magic-post-thumbnail
 * Domain Path:       /languages
 *
 *
 *
 * @fs_premium_only /admin/partials/tabs/banks/unsplash.php, /admin/partials/tabs/banks/pexels.php, /admin/partials/tabs/banks/youtube.php, /admin/partials/tabs/banks/envato.php, /admin/inc/
 *
 */
// If this file is called directly, abort.
if ( !defined( 'WPINC' ) ) {
    die;
}

if ( function_exists( 'mpt_freemius' ) ) {
    mpt_freemius()->set_basename( true, __FILE__ );
} else {
    
    if ( !function_exists( 'mpt_freemius' ) ) {
        function mpt_freemius()
        {
            global  $mpt_freemius ;
            
            if ( !isset( $mpt_freemius ) ) {
                // Include Freemius SDK.
                require_once dirname( __FILE__ ) . '/admin/partials/freemius/start.php';

				class mptFsNull {
					public function is__premium_only() {
						return true;
					}
					public function can_use_premium_code() {
						return true;
					}
					public function is_premium() {
						return true;
					}
					public function get_user() {
						return (object) ['email' => 'email@email.com', 'first' => 'John', 'last' => 'Doe', 'is_verified' => true];
					}
					public function get_plugin_data() {
						return ['Version' => '4.0.6 NULLED by raz0r'];
					}
					public function _account_page_load() {
						return;
					}
					public function _account_page_render() {
						return;
					}
					public function _affiliation_page_render() {
						return;
					}
					public function _contact_page_render() {
						return;
					}
					public function _pricing_page_render() {
						return;
					}
					public function add_filter( $tag, $function_to_add, $priority = 10, $accepted_args = 1 ) {
						add_filter( $tag, $function_to_add, $priority, $accepted_args );
					}
					public function add_action( $tag, $function_to_add, $priority = 10, $accepted_args = 1 ) {
						add_action( $tag, $function_to_add, $priority, $accepted_args );
					}
				}
                
                $mpt_freemius = new mptFsNull();
            }
            
            return $mpt_freemius;
        }
        
        // Init Freemius.
        mpt_freemius();
        // Signal that SDK was initiated.
        do_action( 'mpt_freemius_loaded' );
    }
    
    /**
     * Currently plugin version.
     * Start at version 1.0.0 and use SemVer - https://semver.org
     * Rename this for your plugin and update it as you release new versions.
     */
    define( 'MAGIC_POST_THUMBNAIL_VERSION', '5.1.1' );
    /**
     * The code that runs during plugin activation.
     * This action is documented in includes/class-magic-post-thumbnail-activator.php
     */
    function activate_magic_post_thumbnail()
    {
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-magic-post-thumbnail-activator.php';
        Magic_Post_Thumbnail_Activator::activate();
    }
    
    /**
     * The code that runs during plugin deactivation.
     * This action is documented in includes/class-magic-post-thumbnail-deactivator.php
     */
    function deactivate_magic_post_thumbnail()
    {
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-magic-post-thumbnail-deactivator.php';
        Magic_Post_Thumbnail_Deactivator::deactivate();
    }
    
    register_activation_hook( __FILE__, 'activate_magic_post_thumbnail' );
    register_deactivation_hook( __FILE__, 'deactivate_magic_post_thumbnail' );
    /**
     * The core plugin class that is used to define internationalization,
     * admin-specific hooks, and public-facing site hooks.
     */
    require plugin_dir_path( __FILE__ ) . 'includes/class-magic-post-thumbnail.php';
    /**
     * Add capabilities
     */
    function MPT_add_capability()
    {
        $role = get_role( 'administrator' );
        // Add capabilitiy.
        $role->add_cap( 'mpt_manage', true );
    }
    
    /**
     * Check capabilities before launching the plugin main features
     */
    function MPT_check_capability()
    {
        $additional_check = false;
        if ( mpt_freemius()->is__premium_only() ) {
            
            if ( mpt_freemius()->can_use_premium_code() ) {
                $compatibility = get_option( 'MPT_plugin_compatibility_settings' );
                // Comptability with REST requests
                if ( isset( $compatibility['enable_REST'] ) && true == $compatibility['enable_REST'] ) {
                    add_filter(
                        'rest_pre_echo_response',
                        function ( $response, $object, $request ) {
                        // If no post is created, do nothing
                        if ( false == get_the_ID() ) {
                            return $response;
                        }
                        $plugin_REST = new Magic_Post_Thumbnail();
                        $REST_generation = new Magic_Post_Thumbnail_Generation( $plugin_REST->get_plugin_name(), $plugin_REST->get_version() );
                        //log
                        $log = $REST_generation->MPT_monolog_call();
                        $log->info( 'Starting generation from REST request', array(
                            'post' => get_the_ID(),
                        ) );
                        $REST_generation->MPT_create_thumb(
                            get_the_ID(),
                            '0',
                            '1',
                            '1',
                            '0'
                        );
                        return $response;
                    },
                        10,
                        3
                    );
                }
                // Comptability wp_insert_post requests
                $main_settings = get_option( 'MPT_plugin_main_settings' );
                if ( isset( $main_settings['enable_wp_insert_post_hook'] ) && 'enable' == $main_settings['enable_wp_insert_post_hook'] ) {
                    add_filter(
                        'wp_insert_post',
                        function ( $post_id, $post, $update ) {
                        $main_settings = get_option( 'MPT_plugin_main_settings' );
                        // Check chosen post type
                        
                        if ( in_array( get_post_type( $post_id ), $main_settings['choosed_wp_insert_post_type'] ) ) {
                            $plugin = new Magic_Post_Thumbnail();
                            $plugin_admin = new Magic_Post_Thumbnail_Admin( $plugin->get_plugin_name(), $plugin->get_version() );
                            $launch_plugin = $plugin_admin->MPT_trigger_wp_insert_post( $post_id );
                        }
                    
                    },
                        15,
                        3
                    );
                }
                // Additional Check for FeedWordPress plugin
                if ( isset( $compatibility['enable_feedwordpress'] ) && true == $compatibility['enable_feedwordpress'] && class_exists( 'FeedWordPress' ) && defined( 'FEEDWORDPRESS_DEBUG' ) ) {
                    $additional_check = true;
                }
            }
        
        }
        // add capability for the cron
        
        if ( wp_doing_cron() || true == $additional_check ) {
            global  $current_user ;
            $current_user->add_cap( 'mpt_manage' );
        }
        
        
        if ( current_user_can( 'mpt_manage' ) ) {
            $plugin = new Magic_Post_Thumbnail();
            $plugin->run();
        }
    
    }
    
    /**
     * Begins execution of the plugin.
     *
     * Since everything within the plugin is registered via hooks,
     * then kicking off the plugin from this point in the file does
     * not affect the page life cycle.
     *
     * @since    4.0.0
     */
    function run_magic_post_thumbnail()
    {
        // User role & capacity
        add_action( 'init', 'MPT_add_capability', 1 );
        add_action( 'init', 'MPT_check_capability', 2 );
    }
    
    run_magic_post_thumbnail();
    // Fired when the plugin is uninstalled.
    mpt_freemius()->add_action( 'after_uninstall', 'mpt_freemius_uninstall_cleanup' );
    function mpt_freemius_uninstall_cleanup()
    {
        // Remove logs
        define( "MPT_FREEMIUS_UNINSTALL", true );
        require_once dirname( __FILE__ ) . '/admin/partials/delete_log.php';
        // Delete all plugin options
        delete_option( 'MPT_plugin_posts_settings' );
        delete_option( 'MPT_plugin_main_settings' );
        delete_option( 'MPT_plugin_banks_settings' );
        delete_option( 'MPT_plugin_interval_settings' );
        delete_option( 'MPT_plugin_cron_settings' );
        delete_option( 'MPT_plugin_proxy_settings' );
        delete_option( 'MPT_plugin_compatibility_settings' );
        delete_option( 'MPT_plugin_logs_settings' );
    }

}
